@@ -16,11 +16,19 @@
 package main
 
 import (
+	"fmt"
 	"github.com/osrg/gobgp/gobgp/cmd"
 	"google.golang.org/grpc"
+	"os"
 )
 
+var version = "master"
+
 func main() {
+	if len(os.Args) > 1 && os.Args[1] == "--version" {
+		fmt.Println("gobgp version", version)
+		os.Exit(0)
+	}
 	grpc.EnableTracing = false
 	cmd.NewRootCmd().Execute()
 }
