@@ -79,7 +79,7 @@ func newDynamicPeer(g *config.Global, neighborAddress string, pg *config.PeerGro
 		}).Debugf("Can't overwrite neighbor config: %s", err)
 		return nil
 	}
-	if err := config.SetDefaultNeighborConfigValues(&conf, g.Config.As); err != nil {
+	if err := config.SetDefaultNeighborConfigValues(&conf, pg, g); err != nil {
 		log.WithFields(log.Fields{
 			"Topic": "Peer",
 			"Key":   neighborAddress,
@@ -120,6 +120,10 @@ func NewPeer(g *config.Global, conf *config.Neighbor, loc *table.TableManager, p
 	return peer
 }
 
+func (peer *Peer) AS() uint32 {
+	return peer.fsm.pConf.State.PeerAs
+}
+
 func (peer *Peer) ID() string {
 	return peer.fsm.pConf.State.NeighborAddress
 }
@@ -144,11 +148,19 @@ func (peer *Peer) isGracefulRestartEnabled() bool {
 	return peer.fsm.pConf.GracefulRestart.State.Enabled
 }
 
-func (peer *Peer) isAddPathSendEnabled(family bgp.RouteFamily) bool {
-	if mode, y := peer.fsm.rfMap[family]; y && (mode&bgp.BGP_ADD_PATH_SEND) > 0 {
-		return true
+func (peer *Peer) getAddPathMode(family bgp.RouteFamily) bgp.BGPAddPathMode {
+	if mode, y := peer.fsm.rfMap[family]; y {
+		return mode
 	}
-	return false
+	return bgp.BGP_ADD_PATH_NONE
+}
+
+func (peer *Peer) isAddPathReceiveEnabled(family bgp.RouteFamily) bool {
+	return (peer.getAddPathMode(family) & bgp.BGP_ADD_PATH_RECEIVE) > 0
+}
+
+func (peer *Peer) isAddPathSendEnabled(family bgp.RouteFamily) bool {
+	return (peer.getAddPathMode(family) & bgp.BGP_ADD_PATH_SEND) > 0
 }
 
 func (peer *Peer) isDynamicNeighbor() bool {
@@ -184,7 +196,7 @@ func (peer *Peer) toGlobalFamilies(families []bgp.RouteFamily) []bgp.RouteFamily
 					"Key":    peer.ID(),
 					"Family": f,
 					"VRF":    peer.fsm.pConf.Config.Vrf,
-				}).Warn("invalid family configured for vrfed neighbor")
+				}).Warn("invalid family configured for neighbor with vrf")
 			}
 		}
 		families = fs
@@ -588,17 +600,23 @@ func (peer *Peer) ToConfig(getAdvertised bool) *config.Neighbor {
 	conf := *peer.fsm.pConf
 
 	conf.AfiSafis = make([]config.AfiSafi, len(peer.fsm.pConf.AfiSafis))
-	for i := 0; i < len(peer.fsm.pConf.AfiSafis); i++ {
-		conf.AfiSafis[i] = peer.fsm.pConf.AfiSafis[i]
+	for i, af := range peer.fsm.pConf.AfiSafis {
+		conf.AfiSafis[i] = af
+		conf.AfiSafis[i].AddPaths.State.Receive = peer.isAddPathReceiveEnabled(af.State.Family)
+		if peer.isAddPathSendEnabled(af.State.Family) {
+			conf.AfiSafis[i].AddPaths.State.SendMax = af.AddPaths.State.SendMax
+		} else {
+			conf.AfiSafis[i].AddPaths.State.SendMax = 0
+		}
 	}
 
 	remoteCap := make([]bgp.ParameterCapabilityInterface, 0, len(peer.fsm.capMap))
-	for _, c := range peer.fsm.capMap {
-		for _, m := range c {
+	for _, caps := range peer.fsm.capMap {
+		for _, m := range caps {
 			// need to copy all values here
 			buf, _ := m.Serialize()
-			cap, _ := bgp.DecodeCapability(buf)
-			remoteCap = append(remoteCap, cap)
+			c, _ := bgp.DecodeCapability(buf)
+			remoteCap = append(remoteCap, c)
 		}
 	}
 	conf.State.RemoteCapabilityList = remoteCap
