@@ -57,7 +57,7 @@ func strToASParam(str string) *bgp.PathAttributeAsPath {
 }
 
 func validateOne(tree *radix.Tree, cidr, aspathStr string) config.RpkiValidationResultType {
-	r, _ := ValidatePath(65500, tree, cidr, strToASParam(aspathStr))
+	r := ValidatePath(65500, tree, cidr, strToASParam(aspathStr))
 	return r.Status
 }
 
