@@ -17,6 +17,7 @@ package table
 
 import (
 	"fmt"
+
 	"github.com/osrg/gobgp/packet/bgp"
 )
 
@@ -44,7 +45,7 @@ func (adj *AdjRib) Update(pathList []*Path) {
 			continue
 		}
 		rf := path.GetRouteFamily()
-		key := path.getPrefix()
+		key := fmt.Sprintf("%d:%s", path.GetNlri().PathIdentifier(), path.getPrefix())
 
 		old, found := adj.table[rf][key]
 		if path.IsWithdraw {
@@ -177,7 +178,7 @@ func (adj *AdjRib) Select(family bgp.RouteFamily, accepted bool, option ...Table
 		if d, y := dsts[path.GetNlri().String()]; y {
 			d.knownPathList = append(d.knownPathList, path)
 		} else {
-			dst := NewDestination(path.GetNlri())
+			dst := NewDestination(path.GetNlri(), 0)
 			dsts[path.GetNlri().String()] = dst
 			dst.knownPathList = append(dst.knownPathList, path)
 		}
