@@ -29,13 +29,13 @@ import (
 func TestDestinationNewIPv4(t *testing.T) {
 	peerD := DestCreatePeer()
 	pathD := DestCreatePath(peerD)
-	ipv4d := NewDestination(pathD[0].GetNlri())
+	ipv4d := NewDestination(pathD[0].GetNlri(), 0)
 	assert.NotNil(t, ipv4d)
 }
 func TestDestinationNewIPv6(t *testing.T) {
 	peerD := DestCreatePeer()
 	pathD := DestCreatePath(peerD)
-	ipv6d := NewDestination(pathD[0].GetNlri())
+	ipv6d := NewDestination(pathD[0].GetNlri(), 0)
 	assert.NotNil(t, ipv6d)
 }
 
@@ -88,11 +88,11 @@ func TestCalculate(t *testing.T) {
 	path1.Filter("2", POLICY_DIRECTION_IMPORT)
 	path2.Filter("1", POLICY_DIRECTION_IMPORT)
 
-	d := NewDestination(nlri)
+	d := NewDestination(nlri, 0)
 	d.AddNewPath(path1)
 	d.AddNewPath(path2)
 
-	d.Calculate([]string{"1", "2"}, false)
+	d.Calculate()
 
 	assert.Equal(t, len(d.GetKnownPathList("1")), 0)
 	assert.Equal(t, len(d.GetKnownPathList("2")), 1)
@@ -100,7 +100,7 @@ func TestCalculate(t *testing.T) {
 
 	d.AddWithdraw(path1.Clone(true))
 
-	d.Calculate([]string{"1", "2"}, false)
+	d.Calculate()
 
 	assert.Equal(t, len(d.GetKnownPathList("1")), 0)
 	assert.Equal(t, len(d.GetKnownPathList("2")), 0)
@@ -122,9 +122,9 @@ func TestCalculate2(t *testing.T) {
 	peer1 := &PeerInfo{AS: 1, Address: net.IP{1, 1, 1, 1}}
 	path1 := ProcessMessage(update1, peer1, time.Now())[0]
 
-	d := NewDestination(nlri)
+	d := NewDestination(nlri, 0)
 	d.AddNewPath(path1)
-	d.Calculate(nil, false)
+	d.Calculate()
 
 	// suppose peer2 sends grammaatically correct but semantically flawed update message
 	// which has a withdrawal nlri not advertised before
@@ -134,7 +134,7 @@ func TestCalculate2(t *testing.T) {
 	assert.Equal(t, path2.IsWithdraw, true)
 
 	d.AddWithdraw(path2)
-	d.Calculate(nil, false)
+	d.Calculate()
 
 	// we have a path from peer1 here
 	assert.Equal(t, len(d.knownPathList), 1)
@@ -145,7 +145,7 @@ func TestCalculate2(t *testing.T) {
 	assert.Equal(t, path3.IsWithdraw, false)
 
 	d.AddNewPath(path3)
-	d.Calculate(nil, false)
+	d.Calculate()
 
 	// this time, we have paths from peer1 and peer2
 	assert.Equal(t, len(d.knownPathList), 2)
@@ -156,7 +156,7 @@ func TestCalculate2(t *testing.T) {
 	path4 := ProcessMessage(update4, peer3, time.Now())[0]
 
 	d.AddNewPath(path4)
-	d.Calculate(nil, false)
+	d.Calculate()
 
 	// we must have paths from peer1, peer2 and peer3
 	assert.Equal(t, len(d.knownPathList), 3)
@@ -186,11 +186,11 @@ func TestImplicitWithdrawCalculate(t *testing.T) {
 	path2.Filter("1", POLICY_DIRECTION_IMPORT)
 	path2.Filter("3", POLICY_DIRECTION_IMPORT)
 
-	d := NewDestination(nlri)
+	d := NewDestination(nlri, 0)
 	d.AddNewPath(path1)
 	d.AddNewPath(path2)
 
-	d.Calculate(nil, false)
+	d.Calculate()
 
 	assert.Equal(t, len(d.GetKnownPathList("1")), 0) // peer "1" is the originator
 	assert.Equal(t, len(d.GetKnownPathList("2")), 1)
@@ -209,7 +209,7 @@ func TestImplicitWithdrawCalculate(t *testing.T) {
 	path3.Filter("1", POLICY_DIRECTION_IMPORT)
 
 	d.AddNewPath(path3)
-	d.Calculate(nil, false)
+	d.Calculate()
 
 	assert.Equal(t, len(d.GetKnownPathList("1")), 0) // peer "1" is the originator
 	assert.Equal(t, len(d.GetKnownPathList("2")), 1)
@@ -290,22 +290,22 @@ func TestTimeTieBreaker(t *testing.T) {
 	peer2 := &PeerInfo{AS: 2, LocalAS: 1, Address: net.IP{2, 2, 2, 2}, ID: net.IP{2, 2, 2, 2}} // weaker router-id
 	path2 := ProcessMessage(updateMsg, peer2, time.Now().Add(-1*time.Hour))[0]                 // older than path1
 
-	d := NewDestination(nlri)
+	d := NewDestination(nlri, 0)
 	d.AddNewPath(path1)
 	d.AddNewPath(path2)
 
-	d.Calculate(nil, false)
+	d.Calculate()
 
 	assert.Equal(t, len(d.knownPathList), 2)
 	assert.Equal(t, true, d.GetBestPath("").GetSource().ID.Equal(net.IP{2, 2, 2, 2})) // path from peer2 win
 
 	// this option disables tie breaking by age
 	SelectionOptions.ExternalCompareRouterId = true
-	d = NewDestination(nlri)
+	d = NewDestination(nlri, 0)
 	d.AddNewPath(path1)
 	d.AddNewPath(path2)
 
-	d.Calculate(nil, false)
+	d.Calculate()
 
 	assert.Equal(t, len(d.knownPathList), 2)
 	assert.Equal(t, true, d.GetBestPath("").GetSource().ID.Equal(net.IP{1, 1, 1, 1})) // path from peer1 win
@@ -458,21 +458,22 @@ func TestMultipath(t *testing.T) {
 	updateMsg = bgp.NewBGPUpdateMessage(nil, pathAttributes, nlri)
 	path2 := ProcessMessage(updateMsg, peer2, time.Now())[0]
 
-	d := NewDestination(nlri[0])
+	d := NewDestination(nlri[0], 0)
 	d.AddNewPath(path1)
 	d.AddNewPath(path2)
 
-	best, old, multi := d.Calculate([]string{GLOBAL_RIB_NAME}, false)
-	assert.Equal(t, len(best), 1)
-	assert.Equal(t, old[GLOBAL_RIB_NAME], (*Path)(nil))
+	best, old, multi := d.Calculate().GetChanges(GLOBAL_RIB_NAME, false)
+	assert.NotNil(t, best)
+	assert.Equal(t, old, (*Path)(nil))
 	assert.Equal(t, len(multi), 2)
 	assert.Equal(t, len(d.GetKnownPathList(GLOBAL_RIB_NAME)), 2)
 
 	path3 := path2.Clone(true)
 	d.AddWithdraw(path3)
-	best, old, multi = d.Calculate([]string{GLOBAL_RIB_NAME}, false)
-	assert.Equal(t, len(best), 1)
-	assert.Equal(t, old[GLOBAL_RIB_NAME], path1)
+	dd := d.Calculate()
+	best, old, multi = dd.GetChanges(GLOBAL_RIB_NAME, false)
+	assert.Nil(t, best)
+	assert.Equal(t, old, path1)
 	assert.Equal(t, len(multi), 1)
 	assert.Equal(t, len(d.GetKnownPathList(GLOBAL_RIB_NAME)), 1)
 
@@ -489,8 +490,8 @@ func TestMultipath(t *testing.T) {
 	path4 := ProcessMessage(updateMsg, peer3, time.Now())[0]
 	d.AddNewPath(path4)
 
-	best, _, multi = d.Calculate([]string{GLOBAL_RIB_NAME}, false)
-	assert.Equal(t, len(best), 1)
+	best, _, multi = d.Calculate().GetChanges(GLOBAL_RIB_NAME, false)
+	assert.NotNil(t, best)
 	assert.Equal(t, len(multi), 1)
 	assert.Equal(t, len(d.GetKnownPathList(GLOBAL_RIB_NAME)), 2)
 
@@ -505,10 +506,29 @@ func TestMultipath(t *testing.T) {
 	path5 := ProcessMessage(updateMsg, peer2, time.Now())[0]
 	d.AddNewPath(path5)
 
-	best, _, multi = d.Calculate([]string{GLOBAL_RIB_NAME}, false)
-	assert.Equal(t, len(best), 1)
+	best, _, multi = d.Calculate().GetChanges(GLOBAL_RIB_NAME, false)
+	assert.NotNil(t, best)
 	assert.Equal(t, len(multi), 2)
 	assert.Equal(t, len(d.GetKnownPathList(GLOBAL_RIB_NAME)), 3)
 
 	UseMultiplePaths.Enabled = false
 }
+
+func TestIdMap(t *testing.T) {
+	d := NewDestination(bgp.NewIPAddrPrefix(24, "10.10.0.101"), 64)
+	for i := 0; ; i++ {
+		if id, err := d.localIdMap.FindandSetZeroBit(); err == nil {
+			assert.Equal(t, uint(i+1), id)
+		} else {
+			assert.Equal(t, i, 63)
+			break
+		}
+	}
+	d.localIdMap.Expand()
+	for i := 0; i < 64; i++ {
+		id, _ := d.localIdMap.FindandSetZeroBit()
+		assert.Equal(t, id, uint(64+i))
+	}
+	_, err := d.localIdMap.FindandSetZeroBit()
+	assert.NotNil(t, err)
+}
