@@ -2,7 +2,6 @@
 package table
 
 import (
-	"fmt"
 	"testing"
 	"time"
 
@@ -117,9 +116,8 @@ func TestPathPrependAsnToExistingSeqAttr(t *testing.T) {
 	peer := PathCreatePeer()
 	p := NewPath(peer[0], update.NLRI[0], false, update.PathAttributes, time.Now(), false)
 
-	p.PrependAsn(65000, 1)
+	p.PrependAsn(65000, 1, false)
 	assert.Equal([]uint32{65000, 65001, 65002, 65003, 65004, 65005, 0, 0, 0}, p.GetAsSeqList())
-	fmt.Printf("asns: %v", p.GetAsSeqList())
 }
 
 func TestPathPrependAsnToNewAsPathAttr(t *testing.T) {
@@ -140,7 +138,7 @@ func TestPathPrependAsnToNewAsPathAttr(t *testing.T) {
 	p := NewPath(peer[0], update.NLRI[0], false, update.PathAttributes, time.Now(), false)
 
 	asn := uint32(65000)
-	p.PrependAsn(asn, 1)
+	p.PrependAsn(asn, 1, false)
 	assert.Equal([]uint32{asn}, p.GetAsSeqList())
 }
 
@@ -168,9 +166,8 @@ func TestPathPrependAsnToNewAsPathSeq(t *testing.T) {
 	p := NewPath(peer[0], update.NLRI[0], false, update.PathAttributes, time.Now(), false)
 
 	asn := uint32(65000)
-	p.PrependAsn(asn, 1)
+	p.PrependAsn(asn, 1, false)
 	assert.Equal([]uint32{asn, 0, 0, 0}, p.GetAsSeqList())
-	fmt.Printf("asns: %v", p.GetAsSeqList())
 }
 
 func TestPathPrependAsnToEmptyAsPathAttr(t *testing.T) {
@@ -198,9 +195,8 @@ func TestPathPrependAsnToEmptyAsPathAttr(t *testing.T) {
 	p := NewPath(peer[0], update.NLRI[0], false, update.PathAttributes, time.Now(), false)
 
 	asn := uint32(65000)
-	p.PrependAsn(asn, 1)
+	p.PrependAsn(asn, 1, false)
 	assert.Equal([]uint32{asn, 0, 0, 0}, p.GetAsSeqList())
-	fmt.Printf("asns: %v", p.GetAsSeqList())
 }
 
 func TestPathPrependAsnToFullPathAttr(t *testing.T) {
@@ -237,9 +233,8 @@ func TestPathPrependAsnToFullPathAttr(t *testing.T) {
 	for _, v := range asns {
 		expected = append(expected, uint32(v))
 	}
-	p.PrependAsn(65000, 2)
+	p.PrependAsn(65000, 2, false)
 	assert.Equal(append(expected, []uint32{0, 0, 0}...), p.GetAsSeqList())
-	fmt.Printf("asns: %v", p.GetAsSeqList())
 }
 
 func TestGetPathAttrs(t *testing.T) {
