@@ -337,8 +337,8 @@ func NewPrefix(c config.Prefix) (*Prefix, error) {
 			return nil, fmt.Errorf("mask length range format is invalid")
 		}
 		// we've already checked the range is sane by regexp
-		min, _ := strconv.Atoi(elems[1])
-		max, _ := strconv.Atoi(elems[2])
+		min, _ := strconv.ParseUint(elems[1], 10, 8)
+		max, _ := strconv.ParseUint(elems[2], 10, 8)
 		p.MasklengthRangeMin = uint8(min)
 		p.MasklengthRangeMax = uint8(max)
 	}
@@ -716,25 +716,25 @@ func NewSingleAsPathMatch(arg string) *singleAsPathMatch {
 	onlyRe := regexp.MustCompile("^\\^([0-9]+)\\$$")
 	switch {
 	case leftMostRe.MatchString(arg):
-		asn, _ := strconv.Atoi(leftMostRe.FindStringSubmatch(arg)[1])
+		asn, _ := strconv.ParseUint(leftMostRe.FindStringSubmatch(arg)[1], 10, 32)
 		return &singleAsPathMatch{
 			asn:  uint32(asn),
 			mode: LEFT_MOST,
 		}
 	case originRe.MatchString(arg):
-		asn, _ := strconv.Atoi(originRe.FindStringSubmatch(arg)[1])
+		asn, _ := strconv.ParseUint(originRe.FindStringSubmatch(arg)[1], 10, 32)
 		return &singleAsPathMatch{
 			asn:  uint32(asn),
 			mode: ORIGIN,
 		}
 	case includeRe.MatchString(arg):
-		asn, _ := strconv.Atoi(includeRe.FindStringSubmatch(arg)[1])
+		asn, _ := strconv.ParseUint(includeRe.FindStringSubmatch(arg)[1], 10, 32)
 		return &singleAsPathMatch{
 			asn:  uint32(asn),
 			mode: INCLUDE,
 		}
 	case onlyRe.MatchString(arg):
-		asn, _ := strconv.Atoi(onlyRe.FindStringSubmatch(arg)[1])
+		asn, _ := strconv.ParseUint(onlyRe.FindStringSubmatch(arg)[1], 10, 32)
 		return &singleAsPathMatch{
 			asn:  uint32(asn),
 			mode: ONLY,
@@ -985,8 +985,8 @@ func ParseCommunity(arg string) (uint32, error) {
 	exp := regexp.MustCompile("(\\d+):(\\d+)")
 	elems := exp.FindStringSubmatch(arg)
 	if len(elems) == 3 {
-		fst, _ := strconv.Atoi(elems[1])
-		snd, _ := strconv.Atoi(elems[2])
+		fst, _ := strconv.ParseUint(elems[1], 10, 16)
+		snd, _ := strconv.ParseUint(elems[2], 10, 16)
 		return uint32(fst<<16 | snd), nil
 	}
 	for i, v := range bgp.WellKnownCommunityNameMap {
