@@ -30,6 +30,7 @@ import yaml
 import collections
 
 from lib.base import (
+    wait_for_completion,
     BGPContainer,
     CmdBuffer,
     BGP_ATTR_TYPE_AS_PATH,
@@ -131,6 +132,28 @@ class GoBGPContainer(BGPContainer):
         cmd = '{0} -f {1}/ospfd.conf'.format(daemon_bin, self.QUAGGA_VOLUME)
         self.local(cmd, detach=True)
 
+    def _get_enabled_quagga_daemons(self):
+        daemons = []
+        if self.zebra:
+            daemons.append('zebra')
+            if self.ospfd_config:
+                daemons.append('ospfd')
+        return daemons
+
+    def _is_running(self):
+        return self.local('gobgp global'
+                          ' > /dev/null 2>&1; echo $?', capture=True) == '0'
+
+    def _wait_for_boot(self):
+        for daemon in self._get_enabled_quagga_daemons():
+            def _f_quagga():
+                ret = self.local("vtysh -d {0} -c 'show run' > /dev/null 2>&1; echo $?".format(daemon), capture=True)
+                return ret == '0'
+
+            wait_for_completion(_f_quagga)
+
+        wait_for_completion(self._is_running)
+
     def run(self):
         super(GoBGPContainer, self).run()
         if self.zebra:
@@ -138,6 +161,7 @@ class GoBGPContainer(BGPContainer):
             if self.ospfd_config:
                 self._start_ospfd()
         self._start_gobgp()
+        self._wait_for_boot()
         return self.WAIT_FOR_BOOT
 
     @staticmethod
@@ -341,14 +365,14 @@ class GoBGPContainer(BGPContainer):
         for peer, info in self.peers.iteritems():
             afi_safi_list = []
             if info['interface'] != '':
-                afi_safi_list.append({'config':{'afi-safi-name': 'ipv4-unicast'}})
-                afi_safi_list.append({'config':{'afi-safi-name': 'ipv6-unicast'}})
+                afi_safi_list.append({'config': {'afi-safi-name': 'ipv4-unicast'}})
+                afi_safi_list.append({'config': {'afi-safi-name': 'ipv6-unicast'}})
             else:
                 version = netaddr.IPNetwork(info['neigh_addr']).version
                 if version == 4:
-                    afi_safi_list.append({'config':{'afi-safi-name': 'ipv4-unicast'}})
+                    afi_safi_list.append({'config': {'afi-safi-name': 'ipv4-unicast'}})
                 elif version == 6:
-                    afi_safi_list.append({'config':{'afi-safi-name': 'ipv6-unicast'}})
+                    afi_safi_list.append({'config': {'afi-safi-name': 'ipv6-unicast'}})
                 else:
                     Exception('invalid ip address version. {0}'.format(version))
 
@@ -433,9 +457,8 @@ class GoBGPContainer(BGPContainer):
                                                    'route-reflector-cluster-id': cluster_id}}
 
             if info['addpath']:
-                n['add-paths'] = {'config' : {'receive': True,
-                                              'send-max': 16}}
-
+                n['add-paths'] = {'config': {'receive': True,
+                                             'send-max': 16}}
 
             if len(info.get('default-policy', [])) + len(info.get('policies', [])) > 0:
                 n['apply-policy'] = {'config': {}}
@@ -504,6 +527,7 @@ class GoBGPContainer(BGPContainer):
         c << 'debug zebra packet'
         c << 'debug zebra kernel'
         c << 'debug zebra rib'
+        c << 'ipv6 forwarding'
         c << ''
 
         with open('{0}/zebra.conf'.format(self.quagga_config_dir), 'w') as f:
@@ -529,42 +553,67 @@ class GoBGPContainer(BGPContainer):
             f.writelines(str(c))
 
     def reload_config(self):
-        daemon = ['gobgpd']
-        if self.zebra:
-            daemon.append('zebra')
-            if self.ospfd_config:
-                daemon.append('ospfd')
-        for d in daemon:
-            cmd = '/usr/bin/pkill {0} -SIGHUP'.format(d)
-            self.local(cmd)
-        for v in chain.from_iterable(self.routes.itervalues()):
-            if v['rf'] == 'ipv4' or v['rf'] == 'ipv6':
-                r = CmdBuffer(' ')
-                r << 'gobgp global -a {0}'.format(v['rf'])
-                r << 'rib add {0}'.format(v['prefix'])
-                if v['identifier']:
-                    r << 'identifier {0}'.format(v['identifier'])
-                if v['next-hop']:
-                    r << 'nexthop {0}'.format(v['next-hop'])
-                if v['local-pref']:
-                    r << 'local-pref {0}'.format(v['local-pref'])
-                if v['med']:
-                    r << 'med {0}'.format(v['med'])
-                if v['community']:
-                    r << 'community {0}'.format(
-                        ','.join(v['community'])
-                        if isinstance(v['community'], (list, tuple)) else v['community'])
-                cmd = str(r)
-            elif v['rf'] == 'ipv4-flowspec' or v['rf'] == 'ipv6-flowspec':
-                cmd = 'gobgp global '\
-                      'rib add match {0} then {1} -a {2}'.format(' '.join(v['matchs']), ' '.join(v['thens']), v['rf'])
-            else:
-                raise Exception('unsupported route faily: {0}'.format(v['rf']))
-            self.local(cmd)
+        for daemon in self._get_enabled_quagga_daemons():
+            self.local('pkill {0} -SIGHUP'.format(daemon), capture=True)
+        self.local('pkill gobgpd -SIGHUP', capture=True)
+        self._wait_for_boot()
+
+    def add_route(self, route, rf='ipv4', attribute=None, aspath=None,
+                  community=None, med=None, extendedcommunity=None,
+                  nexthop=None, matchs=None, thens=None,
+                  local_pref=None, identifier=None, reload_config=False):
+        if not self._is_running():
+            raise RuntimeError('GoBGP is not yet running')
+
+        self.routes.setdefault(route, [])
+        path = {
+            'prefix': route,
+            'rf': rf,
+            'attr': attribute,
+            'next-hop': nexthop,
+            'as-path': aspath,
+            'community': community,
+            'med': med,
+            'local-pref': local_pref,
+            'extended-community': extendedcommunity,
+            'identifier': identifier,
+            'matchs': matchs,
+            'thens': thens,
+        }
+
+        c = CmdBuffer(' ')
+        c << 'gobgp global rib -a {0} add'.format(rf)
+        if rf in ('ipv4', 'ipv6'):
+            c << route
+            if path['identifier']:
+                c << 'identifier {0}'.format(path['identifier'])
+            if path['next-hop']:
+                c << 'nexthop {0}'.format(path['next-hop'])
+            if path['local-pref']:
+                c << 'local-pref {0}'.format(path['local-pref'])
+            if path['med']:
+                c << 'med {0}'.format(path['med'])
+            if path['community']:
+                comm = str(path['community'])
+                if isinstance(path['community'], (list, tuple)):
+                    comm = ','.join(path['community'])
+                c << 'community {0}'.format(comm)
+        elif rf.endswith('-flowspec'):
+            c << 'match {0}'.format(' '.join(path['matchs']))
+            c << 'then {0}'.format(' '.join(path['thens']))
+        else:
+            raise Exception('unsupported address family: {0}'.format(rf))
+        self.local(str(c), capture=True)
+
+        self.routes[route].append(path)
 
     def del_route(self, route, identifier=None, reload_config=True):
+        if not self._is_running():
+            raise RuntimeError('GoBGP is not yet running')
+
         if route not in self.routes:
             return
+
         new_paths = []
         for path in self.routes[route]:
             if path['identifier'] != identifier:
@@ -572,11 +621,14 @@ class GoBGPContainer(BGPContainer):
             else:
                 r = CmdBuffer(' ')
                 r << 'gobgp global -a {0}'.format(path['rf'])
-                r << 'rib del {0}'.format(path['prefix'])
+                prefix = path['prefix']
+                if path['rf'].endswith('-flowspec'):
+                    prefix = 'match {0}'.format(' '.join(path['matchs']))
+                r << 'rib del {0}'.format(prefix)
                 if identifier:
                     r << 'identifier {0}'.format(identifier)
                 cmd = str(r)
-                self.local(cmd)
+                self.local(cmd, capture=True)
         self.routes[route] = new_paths
         # no need to reload config
 
