@@ -13,16 +13,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import unittest
-from fabric.api import local
-from lib import base
-from lib.gobgp import *
-from lib.exabgp import *
+from __future__ import absolute_import
+
 import sys
-import os
 import time
+import unittest
+
+from fabric.api import local
 import nose
-from noseplugin import OptionParser, parser_option
+
+from lib.noseplugin import OptionParser, parser_option
+
+from lib import base
+from lib.base import BGP_FSM_ESTABLISHED
+from lib.gobgp import GoBGPContainer
 
 
 class GoBGPTestBase(unittest.TestCase):
@@ -84,7 +88,7 @@ class GoBGPTestBase(unittest.TestCase):
 
         pol0 = {'name': 'pol0', 'statements': [st0]}
 
-        filename = g1.add_policy(pol0, g3, 'in', 'reject')
+        _filename = g1.add_policy(pol0, g3, 'in', 'reject')
 
         g3.add_route('10.0.10.0/24')
         g3.add_route('10.0.20.0/24')
@@ -102,7 +106,7 @@ class GoBGPTestBase(unittest.TestCase):
         time.sleep(1)
 
         num2 = g2.get_neighbor(g1)['state']['messages']['received']['update']
-        self.assertTrue(num+1 == num2)
+        self.assertTrue((num + 1) == num2)
 
         g3.softreset(g1, type='out')
 
@@ -130,9 +134,6 @@ class GoBGPTestBase(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    if os.geteuid() is not 0:
-        print "you are not root."
-        sys.exit(1)
     output = local("which docker 2>&1 > /dev/null ; echo $?", capture=True)
     if int(output) is not 0:
         print "docker not found"
