@@ -43,7 +43,6 @@ class GoBGPTestBase(unittest.TestCase):
         g1 = GoBGPContainer(name='g1', asn=65000, router_id='192.168.0.1',
                             ctn_image_name=gobgp_ctn_image_name,
                             log_level=parser_option.gobgp_log_level)
-
         g2 = GoBGPContainer(name='g2', asn=65001, router_id='192.168.0.2',
                             ctn_image_name=gobgp_ctn_image_name)
         e1 = ExaBGPContainer(name='e1', asn=65002, router_id='192.168.0.3')
@@ -55,8 +54,10 @@ class GoBGPTestBase(unittest.TestCase):
         time.sleep(initial_wait_time)
 
         for cli in cls.clients.values():
-            g1.add_peer(cli, is_rs_client=True, passwd='passwd', passive=True, prefix_limit=10)
-            cli.add_peer(g1, passwd='passwd')
+            # Omit "passwd" to avoid a issue on ExaBGP version 4.0.5:
+            # https://github.com/Exa-Networks/exabgp/issues/766
+            g1.add_peer(cli, is_rs_client=True, passive=True, prefix_limit=10)
+            cli.add_peer(g1)
 
         # advertise a route from route-server-clients
         g2.add_route('10.0.0.0/24')
@@ -66,7 +67,7 @@ class GoBGPTestBase(unittest.TestCase):
 
     # test each neighbor state is turned establish
     def test_01_neighbor_established(self):
-        for cli in self.clients.itervalues():
+        for cli in self.clients.values():
             self.gobgp.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=cli)
 
     def test_02_add_neighbor(self):
